/*
 * Papillon - Solaris security module -  http://www.roqe.org/papillon
 * $Id: papillon.h,v 1.17 2001/11/05 11:27:39 kr Exp $
 *
 * General definitions and structs for the Papillon module.
 */


/*
 * XXX Hiding doesn't work under 64 Bit Systems. 
 */
#ifdef _64BIT
#undef MODHIDING
#endif 


#define SUSER_UID  0
#define SYS_papcomm 180

#define SYS_UNUSED -1
#define DFLT_ARG -1

#ifdef DEBUG
#define dcmn_err(X) cmn_err X
#else
#define dcmn_err(X)
#endif

/*
 * Configuration structure
 */
enum feature { OFF = 0, ON };
enum protection { NONE = 0, WARN, DENY };

typedef struct pap_config {
    enum feature rstproc;
    enum feature ppromisc;
    enum feature modhiding;
    enum feature secstdfd;

    enum protection fifoprot;
    enum protection symprot;
    enum protection hardprot;
} pap_config_t;

typedef struct pap_modfiles {
    char *path;
    vnode_t *vnode;
    vnode_t *parent_vnode;
} pap_modfiles_t;

/*
 * Structure for the syscall redirection
 */
typedef struct pap_syscall {

    int64_t(*sc) ();		/* Original syscall address */
    int64_t(*nc) ();		/* New syscall address */
#ifdef _LP64
    unsigned short flags;	/* Flags */
#else
    unsigned char flags;
#endif
    int index;			/* Syscall index */
    char narg;			/* Number of arguments, -1 = keep default */
    char onarg;			/* Original value */
} pap_syscall_t;

/*
 * Possible request values. 
 */
enum request { GET, SET };

/*
 * Intercepted syscalls
 */
enum syscall_id { LINK = 0, OPEN, OPEN64, EXECVE, PUTMSG, PAPCOMM };

/* 
 * Papillon syscall definitions
 */
int64_t pap_open64(const char *path, int oflag, mode_t mode);
int64_t pap_open(const char *path, int oflag, mode_t mode);
int64_t pap_link(const char *existing, const char *new);
int64_t pap_execve(const char *path, char *const argv[], char *const envp[]);
int64_t pap_putmsg(int fildes, struct strbuf *ctlptr,
		   struct strbuf *dataptr, int *flagsp);
int64_t pap_comm(pap_config_t * c, enum request r);

#ifdef _64BIT
int64_t pap_open64_64(const char *path, int oflag, mode_t mode);
int64_t pap_open_64(const char *path, int oflag, mode_t mode);
int64_t pap_link_64(const char *existing, const char *new);
int64_t pap_execve_64(const char *path, char *const argv[],
		      char *const envp[]);
int64_t pap_putmsg_64(int fildes, struct strbuf *ctlptr,
		      struct strbuf *dataptr, int *flagsp);
#endif

int pap_praccess(struct vnode *vp, int mode, int flags, struct cred *cr);
int pap_vnlookup(struct vnode *dvp, char *nm, struct vnode **vpp,
		 struct pathname *pnp, int flags, struct vnode *rdir,
		 struct cred *);
int pap_vnreaddir(struct vnode *vp, struct uio *uiop, struct cred *cr,
		  int *eofp);

/*
 * Hiding function from hiding.c
 */
void hide_module(void);
