/*
 * Papillon - Solaris security module -  http://www.roqe.org/papillon
 * Copyright (c) 2000,2001 Konrad Rieck <kr@roqe.org>
 * $Id: ppromisc.c,v 1.7 2001/11/02 14:06:56 kr Exp $
 *
 * The idea for this check was introduced by UDP in his module
 * solpromisc-1.0, eventhough I had something similar in my mind. :)
 * 
 */

#define _KERNEL

#if defined(PPROMISC)

#include "headers.h"
#include "papillon.h"
#include "refcnt.h"

extern pap_syscall_t syscalls[];
#ifdef _64BIT
extern pap_syscall_t syscalls64[];
#endif

extern pap_config_t config;
extern kmutex_t promisc_lock;
extern krwlock_t config_rwlock;

void check_promisc(int fildes, dl_promiscon_req_t * promiscon)
{
    char *device;
    file_t *file;
    cred_t *cred;
    dev_t rdev;

    if (promiscon->dl_primitive == DL_PROMISCON_REQ &&
	promiscon->dl_level == DL_PROMISC_PHYS) {

	file = getf(fildes);

	if (!file)
	    return;

	rdev = file->f_vnode->v_rdev;
	device = ddi_major_to_name(getmajor(rdev));
	cred = ddi_get_cred();

	cmn_err(CE_WARN, "Promiscuous flag on %s, process "
		"%s (pid %d, uid %d)",
		device ? device : "unknown", u.u_psargs,
		ddi_get_pid(), cred->cr_ruid);

	releasef(fildes);
    }

    return;
}

int64_t pap_putmsg(int fildes, struct strbuf * ctlptr, struct strbuf
		   * dataptr, int *flagsp)
{
    struct strbuf kctlptr;
    dl_promiscon_req_t *kpromiscon;

    inc_refcnt();

    dcmn_err((CE_CONT, "putmsg() syscall.\n"));

    rw_enter(&config_rwlock, RW_READER);

    if (!config.ppromisc ||
	copyin((void *) ctlptr, (void *) &kctlptr, sizeof(struct strbuf)))
	goto err;

    mutex_enter(&promisc_lock);

    kpromiscon = (dl_promiscon_req_t *) kmem_alloc(kctlptr.len, KM_SLEEP);
    if (!kpromiscon)
	goto err_and_unlock;

    if (copyin((void *) kctlptr.buf, (void *) kpromiscon, kctlptr.len))
	goto err_and_free;

    check_promisc(fildes, kpromiscon);

  err_and_free:
    kmem_free(kpromiscon, kctlptr.len);
  err_and_unlock:
    mutex_exit(&promisc_lock);
  err:

    rw_exit(&config_rwlock);
    dec_refcnt();

    return syscalls[PUTMSG].sc(fildes, ctlptr, dataptr, flagsp);
}

#ifdef _64BIT

int64_t pap_putmsg_64(int fildes, struct strbuf * ctlptr, struct strbuf
		      * dataptr, int *flagsp)
{
    struct strbuf kctlptr;
    dl_promiscon_req_t *kpromiscon;

    inc_refcnt();

    dcmn_err((CE_CONT, "putmsg() syscall.\n"));

    rw_enter(&config_rwlock, RW_READER);

    if (!config.ppromisc ||
	copyin((void *) ctlptr, (void *) &kctlptr, sizeof(struct strbuf)))
	goto err;

    mutex_enter(&promisc_lock);

    kpromiscon = (dl_promiscon_req_t *) kmem_alloc(kctlptr.len, KM_SLEEP);
    if (!kpromiscon)
	goto err_and_unlock;

    if (copyin((void *) kctlptr.buf, (void *) kpromiscon, kctlptr.len))
	goto err_and_free;

    check_promisc(fildes, kpromiscon);

  err_and_free:
    kmem_free(kpromiscon, kctlptr.len);
  err_and_unlock:
    mutex_exit(&promisc_lock);
  err:

    rw_exit(&config_rwlock);
    dec_refcnt();

    return syscalls64[PUTMSG].sc(fildes, ctlptr, dataptr, flagsp);
}

#endif				/* _64BIT */

#endif				/* PPROMISC */
